@@ -1,8 +1,7 @@
 from smot.classes import Node
-from smot.util import log, die
+from smot.util import die
 from collections import Counter, defaultdict
 import re
-import sys
 import math
 import random
 
@@ -47,9 +46,11 @@ def treepull(node, fun, **kwargs):
     fun :: NodeData -> [NodeData] -> NodeData
     """
     if node.data.isLeaf:
-        node.data = fun(node.data, [])
+        node.data = fun(node.data, [], **kwargs)
     else:
-        node.kids = [treepull(kid, fun, **kwargs) for kid in node.kids if kid is not None]
+        node.kids = [
+            treepull(kid, fun, **kwargs) for kid in node.kids if kid is not None
+        ]
         node.data = fun(node.data, [kid.data for kid in node.kids], **kwargs)
     return node
 
@@ -68,13 +69,16 @@ def treepush(node, fun, **kwargs):
 
     return node
 
+
 def tips(node):
     def _fun(b, x):
         if x.isLeaf:
             b.append(x.label)
         return b
+
     return treefold(node, _fun, [])
 
+
 def partition(xs, f):
     a = []
     b = []
@@ -123,13 +127,15 @@ def clean(node, isRoot=True):
 def factorByLabel(node, fun, **kwargs):
     """
     Assign factors to nodes based on the node label string
+
+    kwargs are passed to the `fun` within the `mapfun` function.
     """
 
     def mapfun(ndata, **kwargs):
         ndata.factor = fun(ndata.label, **kwargs)
         return ndata
 
-    return treemap(node, mapfun, **kwargs)
+    return setFactorCounts(treemap(node, mapfun, **kwargs))
 
 
 def factorByField(node, field, default=None, sep="|"):
@@ -171,8 +177,7 @@ def factorByTable(node, filename, default=None):
                 (k, v) = line.strip().split("\t")
                 factorMap[k] = v
             except ValueError:
-                print("Expected two columns in --factor-by-table file", file=sys.stderr)
-                sys.exit(1)
+                die("Expected two columns in --factor-by-table file")
 
         def _fun(name):
             if name:
@@ -199,7 +204,7 @@ def getFactor(node):
         return None
 
 
-def imputeFactors(node):
+def imputeMonophyleticFactors(node):
     def setFactors(node, factor):
         def _fun(b):
             b.factor = factor
@@ -210,7 +215,7 @@ def imputeFactors(node):
     if node.data.factorCount and isMonophyletic(node):
         node = setFactors(node, getFactor(node))
     else:
-        newkids = [imputeFactors(kid) for kid in node.kids]
+        newkids = [imputeMonophyleticFactors(kid) for kid in node.kids]
         node.kids = newkids
     return node
 
@@ -274,7 +279,6 @@ def sampleN(node, n):
         return node
     if not node.kids and not n == 1:
         raise "Bug in sampleN"
-    N = sum([kid.data.nleafs for kid in node.kids])
     selection = distribute(n, len(node.kids), [kid.data.nleafs for kid in node.kids])
     node.kids = [sampleN(kid, m) for kid, m in zip(node.kids, selection) if m > 0]
     if len(node.kids) == 1:
@@ -365,7 +369,7 @@ def distribute(count, groups, sizes=None):
 
 def setNLeafs(node):
     if not node:
-      return node
+        return node
     if node.data.isLeaf:
         node.data.nleafs = 1
     else:
@@ -389,9 +393,9 @@ def setFactorCounts(node):
     return node
 
 
-def sampleContext(node, keep=[], maxTips=5):
+def sampleBalanced(node, keep=[], maxTips=5):
     # recursive sampler
-    def _sampleContext(node):
+    def _sampleBalanced(node):
         newkids = []
         for kid in node.kids:
             if (
@@ -403,17 +407,24 @@ def sampleContext(node, keep=[], maxTips=5):
                 else:
                     newkids.append(sampleN(kid, maxTips))
             else:
-                newkids.append(_sampleContext(kid))
+                newkids.append(_sampleBalanced(kid))
         node.kids = newkids
         return node
 
     node = setNLeafs(node)
     node = setFactorCounts(node)
-    return clean(_sampleContext(node))
+    return clean(_sampleBalanced(node))
 
 
 def sampleParaphyletic(
-    node, proportion=None, scale=None, number=None, keep=[], keep_regex="", minTips=3, seed=None
+    node,
+    proportion=None,
+    scale=None,
+    number=None,
+    keep=[],
+    keep_regex="",
+    minTips=1,
+    seed=None,
 ):
     rng = random.Random(seed)
 
@@ -431,7 +442,6 @@ def sampleParaphyletic(
 
         def _sample(labels):
             return min(len(labels), number)
-            
 
     def getLabels(node):
         def _collect(b, d):
@@ -455,8 +465,7 @@ def sampleParaphyletic(
             try:
                 sample = rng.sample(sorted(list(samplers)), N)
             except ValueError:
-                log(f"Bad sample size ({N}) for population of size ({len(labels)})")
-                sys.exit(1)
+                die(f"Bad sample size ({N}) for population of size ({len(labels)})")
             return sample + keepers
 
     def _select(node, selected, paraGroup, paraFactor):
@@ -527,8 +536,15 @@ def sampleParaphyletic(
     return clean(treecut(node, _cull))
 
 
-def sampleProportional(
-    node, proportion=None, scale=None, number=None, keep=[], keep_regex="", minTips=3, seed=None
+def sampleMonophyletic(
+    node,
+    proportion=None,
+    scale=None,
+    number=None,
+    keep=[],
+    keep_regex="",
+    minTips=1,
+    seed=None,
 ):
     rng = random.Random(seed)
 
@@ -548,21 +564,21 @@ def sampleProportional(
         return sampleRandom(node=node, rng=rng, count_fun=count_fun, keep_fun=keep_fun)
 
     # recursive sampler
-    def _sampleProportional(node):
+    def _sampleMonophyletic(node):
         nfactors = len(node.data.factorCount)
-        if(nfactors == 0):
-          return _sample(node)
+        if nfactors == 0:
+            return _sample(node)
         elif nfactors == 1:
-          if list(node.data.factorCount.keys())[0] in keep:
-            return node
-          else:
-            node = _sample(node)
+            if list(node.data.factorCount.keys())[0] in keep:
+                return node
+            else:
+                node = _sample(node)
         else:
-            node.kids = [_sampleProportional(kid) for kid in node.kids]
+            node.kids = [_sampleMonophyletic(kid) for kid in node.kids]
         return node
 
     node = setFactorCounts(node)
-    return clean(_sampleProportional(node))
+    return clean(_sampleMonophyletic(node))
 
 
 def colorTree(node, color):
@@ -585,8 +601,8 @@ def colorMono(node, colormap):
 
 def filterMono(node, condition, action):
     if len(node.data.factorCount) == 1:
-      if condition(node):
-        node = action(node)
+        if condition(node):
+            node = action(node)
     else:
         node.kids = [filterMono(kid, condition, action) for kid in node.kids]
     return node
@@ -611,7 +627,6 @@ def colorPara(node, colormap):
         common = intersectionOfSets([k.data.factorCount.keys() for k in node.kids])
         if len(common) == 1:
             try:
-                color = colormap[list(common)[0]]
                 node = colorTree(node, colormap[list(common)[0]])
             except KeyError:
                 pass
