@@ -74,6 +74,13 @@ class ListOfStringsType(click.ParamType):
 
 ListOfStrings = ListOfStringsType()
 
+dec_default = click.option(
+    "--default",
+    type=str,
+    default=None,
+    help="The name to assign to tips that do not match a factor",
+)
+
 
 def factorTree(
     node,
@@ -93,28 +100,24 @@ def factorTree(
             die(
                 f"""Expected a positive integer for field --factor-by-field, got '{factor_by_field}'"""
             )
-        nodes = alg.factorByField(node, field, default=default)
+        node = alg.factorByField(node, field, default=default)
     elif factor_by_capture is not None:
         node = alg.factorByCapture(node, pat=factor_by_capture, default=default)
     elif factor_by_table is not None:
         node = alg.factorByTable(node, filename=factor_by_table, default=default)
-    node = alg.setFactorCounts(node)
 
     if patristic:
         node = alg.imputePatristicFactors(node)
     elif impute:
-        node = alg.imputeFactors(node)
+        node = alg.imputeMonophyleticFactors(node)
 
     return node
 
 
 def read_tree(treefile):
-    from smot.parser import p_tree
+    from smot.parser import read_fh
 
-    rawtree = treefile.readlines()
-    rawtree = "".join(rawtree)
-    tree = p_tree.parse(rawtree)
-    return tree
+    return read_fh(treefile)
 
 
 dec_tree = click.argument("TREE", default=sys.stdin, type=click.File())
@@ -137,6 +140,22 @@ def tips(tree):
         print(tip)
 
 
+@click.command()
+@dec_tree
+def stat(tree):
+    """
+    Display statistics for an input tree
+    """
+    import smot.algorithm as alg
+
+    tree = read_tree(tree)
+    tree.tree = alg.setNLeafs(tree.tree)
+
+    tips = list(alg.tips(tree.tree))
+
+    print(len(tips))
+
+
 def factoring(function):
     function = click.option(
         "--factor-by-capture",
@@ -152,7 +171,9 @@ def factoring(function):
     )(function)
 
     function = click.option(
-        "--factor-by-table", type=MaybeString, help="I don't even know what this is"
+        "--factor-by-table",
+        type=MaybeString,
+        help="Read factors from a TAB-delimited file (tip labels in the first column and factors in the second column)",
     )(function)
 
     return function
@@ -231,9 +252,7 @@ dec_patristic = click.option(
 @click.command()
 @factoring
 @dec_keep
-@click.option(
-    "--default", type=str, help="The name to assign to tips that do not match a factor"
-)
+@dec_default
 @dec_max_tips
 @click.option("--zero", is_flag=True, help="Set branches without lengths to 0")
 @dec_newick
@@ -272,7 +291,7 @@ def equal(
         factor_by_table=factor_by_table,
         default=default,
     )
-    tree.tree = alg.sampleContext(tree.tree, keep=keep, maxTips=max_tips)
+    tree.tree = alg.sampleBalanced(tree.tree, keep=keep, maxTips=max_tips)
 
     if newick:
         print(sf.newick(tree))
@@ -284,9 +303,7 @@ def equal(
 @factoring
 @dec_keep
 @dec_keep_regex
-@click.option(
-    "--default", type=str, help="The name to assign to tips that do not match a factor"
-)
+@dec_default
 @dec_min_tips
 @dec_proportion
 @dec_scale
@@ -295,7 +312,7 @@ def equal(
 @dec_newick
 @click.option("--zero", is_flag=True, help="Set branches without lengths to 0")
 @dec_tree
-def prop(
+def mono(
     factor_by_capture,
     factor_by_field,
     factor_by_table,
@@ -330,7 +347,7 @@ def prop(
         factor_by_table=factor_by_table,
         default=default,
     )
-    tree.tree = alg.sampleProportional(
+    tree.tree = alg.sampleMonophyletic(
         tree.tree,
         keep=keep,
         keep_regex=keep_regex,
@@ -351,9 +368,7 @@ def prop(
 @factoring
 @dec_keep
 @dec_keep_regex
-@click.option(
-    "--default", type=str, help="The name to assign to tips that do not match a factor"
-)
+@dec_default
 @dec_min_tips
 @dec_proportion
 @dec_scale
@@ -423,12 +438,7 @@ def para(
     "method", type=click.Choice(["table", "prepend", "append"], case_sensitive=False)
 )
 @factoring
-@click.option(
-    "--default",
-    type=str,
-    default=None,
-    help="The name to assign to tips that do not match a factor",
-)
+@dec_default
 @dec_impute
 @dec_patristic
 @dec_newick
@@ -526,6 +536,7 @@ def tipsed(pattern, replacement, newick, tree):
 
     tree = read_tree(tree)
     tree.tree = alg.treemap(tree.tree, fun_)
+    tree.colmap = {re.sub(pat, replacement, k): v for (k, v) in tree.colmap.items()}
 
     if newick:
         print(sf.newick(tree))
@@ -569,7 +580,7 @@ def grep(pattern, tree, invert_match, perl, newick, file):
             matcher = lambda s: re.search(regex, s)
     else:
         if invert_match:
-            matcher = lambda s: not pattern in s
+            matcher = lambda s: pattern not in s
         else:
             matcher = lambda s: pattern in s
 
@@ -628,6 +639,7 @@ def grep(pattern, tree, invert_match, perl, newick, file):
     help="Replace arg1 with arg2 in all leaf names in group if all conditions are met",
 )
 @factoring
+@dec_default
 @dec_patristic
 @dec_seed
 @dec_newick
@@ -648,6 +660,7 @@ def filter_cmd(
     factor_by_capture,
     factor_by_field,
     factor_by_table,
+    default,
     # phylogenetic options
     patristic,
     seed,
@@ -669,6 +682,7 @@ def filter_cmd(
         factor_by_capture=factor_by_capture,
         factor_by_field=factor_by_field,
         factor_by_table=factor_by_table,
+        default=default,
         patristic=patristic,
     )
 
@@ -705,7 +719,7 @@ def filter_cmd(
     elif color:
         action = lambda x: alg.colorTree(x, color)
     elif sample:
-        action = lambda x: sampleProportional(
+        action = lambda x: alg.sampleMonophyletic(
             x, proportion=sample, scale=None, minTips=3, keep_regex="", seed=seed
         )
     elif replace:
@@ -820,9 +834,14 @@ def colorBranches(
         with open(colormap, "r") as f:
             try:
                 _colormap = {
-                    f.strip(): c.strip()
-                    for (f, c) in [p.split("\t") for p in f.readlines()]
+                    f.strip(): c.strip().upper()
+                    for (f, c) in [p.strip().split("\t") for p in f.readlines()]
                 }
+                for clade, color in _colormap.items():
+                    if color[0] != "#":
+                        _colormap[clade] = "#" + color
+                    if len(color) != 7:
+                        die('Expected colors in hexadecimal (e.g., "#AA10FF")')
             except ValueError:
                 die("Invalid color map: expected TAB-delimited, two-column file")
     else:
@@ -890,11 +909,76 @@ def rm_color(newick, tree):
     else:
         print(sf.nexus(tree))
 
+# Remove all black color
+def unblack(x, colmap):
+  if "!color" in x.form and x.form["!color"] == "#000000":
+    del x.form["!color"]
+  if x.isLeaf and x.label in colmap and colmap[x.label] == "#000000":
+    del colmap[x.label]
+  return x
+
+# color nodes by tip
+def tip2node(x, kids, colmap):
+    child_node_colors = [
+        kid.form["!color"]
+        for kid in kids
+        if "!color" in kid.form and kid.form["!color"]
+    ]
+    child_leaf_colors = [colmap[kid.label] for kid in kids if kid.isLeaf and kid.label in colmap]
+    child_colors = set(child_node_colors + child_leaf_colors)
+    if len(child_colors) == 1:
+        x.form["!color"] = list(child_colors)[0]
+    return x
+
+# color tips by node
+def node2tip(x, kid, colmap):
+    if "!color" in x.form:
+      if not "!color" in kid.form:
+          kid.form["!color"] = x.form["!color"]
+      if kid.isLeaf:
+          colmap[kid.label] = x.form["!color"]
+    return kid
+
+
+@click.command(name="pull")
+@dec_tree
+def pull_color(tree):
+    "Pull colors from tips to nodes"
+
+    import smot.algorithm as alg
+
+    tree = read_tree(tree)
+
+    colmap = tree.colmap
+
+    tree.tree = alg.treemap(tree.tree, unblack, colmap=colmap)
+    tree.tree = alg.treepull(tree.tree, tip2node, colmap=colmap)
+    tree.tree = alg.treepush(tree.tree, node2tip, colmap=colmap)
+
+    print(sf.nexus(tree))
+
+@click.command(name="push")
+@dec_tree
+def push_color(tree):
+    "Push colors from nodes to tips"
+
+    import smot.algorithm as alg
+
+    tree = read_tree(tree)
+
+    colmap = tree.colmap
+
+    tree.tree = alg.treemap(tree.tree, unblack, colmap=colmap)
+    tree.tree = alg.treepush(tree.tree, node2tip, colmap=colmap)
+
+    print(sf.nexus(tree))
+
 
 CONTEXT_SETTINGS = dict(help_option_names=["-h", "--help"])
 
 
 @click.group(help="Simple Manipulation Of Trees", context_settings=CONTEXT_SETTINGS)
+@click.version_option(__version__, "-v", "--version", message=__version__)
 def cli():
     pass
 
@@ -917,7 +1001,7 @@ def sample():
 
 
 sample.add_command(equal)
-sample.add_command(prop)
+sample.add_command(mono)
 sample.add_command(para)
 
 
@@ -954,8 +1038,11 @@ def color():
 color.add_command(leaf)
 color.add_command(branch)
 color.add_command(rm_color)
+color.add_command(push_color)
+color.add_command(pull_color)
 
 cli.add_command(tips)
+cli.add_command(stat)
 cli.add_command(sample)
 cli.add_command(factor)
 cli.add_command(tipsed)
@@ -970,6 +1057,6 @@ def main():
 
 
 if __name__ == "__main__":
-    if os.name is "posix":
+    if os.name == "posix":
         signal.signal(signal.SIGPIPE, signal.SIG_DFL)
     main()
