@@ -1,10 +1,22 @@
 import parsec as p
 import re
-from smot.util import concat, rmNone, die
+from smot.util import die
 from smot.classes import Node, Tree
 
 p_whitespace = p.regex(r"\s*", re.MULTILINE)
 
+def read_fh(treefh):
+    rawtree = treefh.readlines()
+    rawtree = "".join(rawtree)
+    tree = p_tree.parse(rawtree)
+    return tree
+
+def read_file(treefile):
+    with open(treefile, "r") as treefh:
+        return read_fh(treefh)
+
+def read_text(treestr):
+   return p_tree.parse(treestr) 
 
 def p_parens(parser):
     return p.string("(") >> parser << p.string(")")
@@ -127,7 +139,7 @@ def p_nexus_tree_line():
 def make_tip_color_map(xs):
     color_map = dict()
     for (name, form) in xs:
-        if not form is None and "!color" in form:
+        if form is not None and "!color" in form:
             color_map[name] = form["!color"]
     return color_map
 
@@ -140,7 +152,7 @@ def dieIfMultiple(xs):
         return xs[0]
     elif len(xs) == 0:
         die(
-            f"Failed to parse tree. The tree may be in an unsupported format (only Nexus and Newick are supported) or the tip labels may have strange characters or escape conventions. If you are sure this is a valid tree, send it to the maintainer and ask them to fix the smot parser."
+            "Failed to parse tree. The tree may be in an unsupported format (only Nexus and Newick are supported) or the tip labels may have strange characters or escape conventions. If you are sure this is a valid tree, send it to the maintainer and ask them to fix the smot parser."
         )
     else:
         die(f"Expected a single entry in this NEXUS file, found {len(xs)}")
